
' *******************************************************************************************************
' *                                                                                                     *
' *     Ethernet_Demo_003.spin                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

' Demo Description
' ================
'
' Demonstration of a bit-banged implementation of 10base-T Ethernet to allow the Propeller Chip to send
' IP/UDP packets over ethernet using minimal PHY hardware ( no magnetics ).
'
' Status
' ======
'
' It's alive ! Coughing and spluttering but shown to work. UDP Packets sent are reported as received by
' a packet sniffer.
'
' It seems necessary to change the software Manchester encoding from 0->10/1->01 to 0->01/1->10 every
' now and again to get packets through.
'
' Proven to work partially only with P1=TX+ / P2=TX-, circuit as shown below, using software Manchester
' encoding. Not all packets get through, and the hub Rx Led flashes repeatedly and drifts in and out of
' collision detected. 
'
' Hardware encoding does not work and needs a re-think for implementation; with chroma on and the current
' mechanism used, it's not possible to send a positive only NLP, only a Manchester Enooded NLP which is
' not what's needed.
'
' Known Bugs, Issues and Features
' ===============================
'
' See status comments above
' Not fully tested
' Unreliable operation
' Hardware Manchester encoding not implemented / does not work
' Requires 80MHz CLKFREQ
' TV_BASE_PIN ( for debug TV display ) set to P12. May need changing
' ETHERNET_BASE_PIN set for P1 ( P1=TX+ and P2=TX- ). May need changing
' Parallax's TV_Text.spin uses NTSC. May need changing for PAL users
'
' Choosing an ETHERNET_BASE_PIN
' =============================
'
' The pin specified must be the first of a consecutive pair within a byte group. The pin specified
' will be TX+ the next consecutive pin will be TX-.
'
' To use hardware Manchester encoding a certain number of odd-numbered pins can be used as TX+;
' P1, P5, P9, P13, P17, P21, P25, P29, and the next consecutive pin ( P2, P6, P10 etc ) will be 
' the TX- pin of the pair.
'
' If hardware Manchester encoding has been disabled in Ethernet_Driver_???.spin then the software
' will auto re-configure to perform software Manchester encoding on the pins as specified. Unless
' the pin chosen is one of the eight specified above and hardware Manchester encoding has been 
' enabled then software Manchester encoding wil be used.
'
' Pin pairs which can be used for ethernet transmission are -
'
'     TX+ : 0 1 2 3 4 5 6 - 8  9 10 11 12 13 14 - 16 17 18 19 20 21 22 - 24 25 26 27 28 29 30
'     TX- : 1 2 3 4 5 6 7 - 9 10 11 12 13 14 15 - 17 18 19 20 21 22 23 - 25 26 27 28 29 30 31   
'
'     Notes   H       H        H          H           H           H          H     *  * H*  *
'
'     H = Hardware Manchester encoding possible
'     * = May be incompatible with other Propeller functions ( Eeprom / Programming )    
'
' Hardware
' ========
'
' NOTE : ***** THIS HARDWARE IS NOT RECOMMENDED OR APPROVED *****
'
'   Propeller                                          View into RJ45 Socket
' .-----------.      68R             RJ45 Skt         .---------------------.
' |           |      ___                              | .-.-.-.-.-.-.-.-.-. |
' |    TX+ P1 |-----|___|----------> Pin 1            | | V V V V V V V V | |
' |           |                                       | | 1 2 3 4 5 6 7 8 | |
' |           |      ___                              | |                 | |   
' |    TX- P2 |-----|___|----||----> Pin 2            | `------.   .------' |
' |           |                                       |        `---'        |
' `-----------'      68R    100nF                     `---------------------'
'
' Pin    PC     Hub     T-568A            T-568B            Pairs
'
'  1     TX+    RX+     Green / White     Orange / White    ------.
'  2     TX-    RX-     Green             Orange            ------'
'  3     RX+    TX+     Orange / White    Green / White     ------.
'  4                    Blue              Blue              ---.  |
'  5                    Blue / White      Blue / White      ---'  |
'  6     RX-    TX-     Orange            Green             ------'
'  7                    Brown / White     Brown / White     ---.
'  8                    Brown             Brown             ---'
'
' Use straight-through cable to connect to Hub
' Use cross-over cable to connect to PC

' History
' =======
'
' 003   Requires Ethernet_003.spin
'       Pin allocation switched (TX+/TX-)
'       Default pins changed to P1=TX+ / P2=TX-
'       Added 100nF to hardware interface
'
' 002   Requires Ethernet_002.spin
'       Added Credits and Acknowledgements
'
' 001   First public release

' Credits and Acknowledgements
' ============================
'
' Thanks to Ariba (Andy) of the Propeller Forum for details of video generation settings.
'
' Thanks to deSilva of the Propeller Forum and his Assembly Language Tutorial tutorial for details
' of the video generation mechanism used by the Propeller Chip.
'
' Thanks to Phil Pilgrim (PhiPi) of the Propeller Forum for provision of test code to do hardware
' Manchester encoding using Broadband Video plus Chroma.
'
' Inspiration for this project came from ...
'
' http://www.fpga4fun.com/10BASE-T.html
'
'    Jean P. Nicolle : "10BASE-T FPGA interface"
'
' http://www.cesko.host.sk/IgorPlugUDP/IgorPlug-UDP%20(AVR)_eng.htm
'
'     Ing. Igor Cesko : "Implementation UDP into microcontroller Igor Atmel-UDP device: UDP/IP
'     infrared remote control"
'
' http://www.national.com/an/AN/AN-1519.pdf
'
'    National Semiconductor Application Note 1519. David Miller, Devin Seely, Tung Ngo : "DP83848
'    PHYTER Transformerless Ethernet Operation"

CON

  _CLKMODE          = XTAL1 + PLL16x
  _XINFREQ          = 5_000_000

  TV_BASE_PIN       = 12
  ETHERNET_BASE_PIN = 1                                 ' P1 = TX+ / P2 = TX-

  UDP_UNIT          = 1                                 ' Must be unique for every source ( 0..255 )
  UDP_PORT          = 1024                              ' Must be same as receiver
  
OBJ

  Ethernet : "Ethernet_003"

PUB Main | packetNumber

  Ethernet.Start( ETHERNET_BASE_PIN , TV_BASE_PIN )

  ' Set Source
  
  ' Source MAC Address = $AA $BB $CC $DD $EE $FF
  '                      -----.----- -----.-----
  '                           |           |
  '                           |           `-------------- Serial Number
  '                           |
  '                           `-------------------------- Vendor / Manufacturer's ID ( OUI )
  '
  '                      %AAAAAAAA
  '                             ||
  '                             |`----------------------- Group / Multicast Bit ( for source must be 0 )
  '                             `------------------------ 1 = Locally Administered / 0 = Burned-In
  
  Ethernet.SetSrcMac($02,$00,$00,$00,$00,UDP_UNIT)      ' Locally Administered MAC Address
  Ethernet.SetSrcAdr(169,254,0,UDP_UNIT)                ' Auto-Configured IP Address ( RFC 3330 )                      ' 
  Ethernet.SetSrcPrt(UDP_PORT)                          ' Port

 ' Set Destination

  Ethernet.SetDstMac($FF,$FF,$FF,$FF,$FF,$FF)           ' Broadcast MAC Address
  Ethernet.SetDstAdr(255,255,255,255)                   ' Broadcast IP Address
  Ethernet.SetDstPrt(UDP_PORT)                          ' Port

  repeat  

    Ethernet.CreateEmptyUdpPacket                       ' Start a new UDP Packet

    ' Put some meaningful data into the UDP payload ...
    
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_0 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_1 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_2 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_3 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_4 ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#ETH_SRC_MAC_5 ) )

    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#IP_SRC_ADR_0  ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#IP_SRC_ADR_1  ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#IP_SRC_ADR_2  ) )
    Ethernet.AddUdpData( Ethernet.GetBytePacketByte( Ethernet#IP_SRC_ADR_3  ) )

    ' An incrementing count always helps things along ...

    packetNumber := Ethernet.GetPacketNumber
    
    Ethernet.AddUdpData( packetNumber >> 24 )            
    Ethernet.AddUdpData( packetNumber >> 16 )
    Ethernet.AddUdpData( packetNumber >> 8  )
    Ethernet.AddUdpData( packetNumber       )

    Ethernet.SendUdpPacket                              ' Send the Packet

    PauseMs( 5000 )                                     ' Send one every five seconds

PRI PauseMs( mS )

  waitcnt( clkfreq / 1000 * mS + CNT )
  