
' *******************************************************************************************************
' *                                                                                                     *
' *     Ethernet_Driver_003.spin                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

' Known Bugs, Issues and Features
' ===============================
'
' BUG : ***** HARDWARE MANCHESTER ENCODING NOT WORKING YET *****

' Requires 80MHz CLKFREQ
' No check on CLKFREQ
' Should determine FRQA and PLLDIV for CTRA from CLKFREQ
' Cannot send 100nS NLP with Chroma on for hardware Manchester Encoding, Needs redesign.
'
' Hardware
' ========
'
' NOTE : ***** THIS HARDWARE IS NOT RECOMMENDED OR APPROVED *****
'
'   Propeller                                          View into RJ45 Socket
' .-----------.      68R             RJ45 Skt         .---------------------.
' |           |      ___                              | .-.-.-.-.-.-.-.-.-. |
' |    TX+ P1 |-----|___|----------> Pin 1            | | V V V V V V V V | |
' |           |                                       | | 1 2 3 4 5 6 7 8 | |
' |           |      ___                              | |                 | |   
' |    TX- P2 |-----|___|----||----> Pin 2            | `------.   .------' |
' |           |                                       |        `---'        |
' `-----------'      68R    100nF                     `---------------------'
'
' Pin    PC     Hub     T-568A            T-568B            Pairs
'
'  1     TX+    RX+     Green / White     Orange / White    ------.
'  2     TX-    RX-     Green             Orange            ------'
'  3     RX+    TX+     Orange / White    Green / White     ------.
'  4                    Blue              Blue              ---.  |
'  5                    Blue / White      Blue / White      ---'  |
'  6     RX-    TX-     Orange            Green             ------'
'  7                    Brown / White     Brown / White     ---.
'  8                    Brown             Brown             ---'
'
' Use straight-through cable to connect to Hub
' Use cross-over cable to connect to PC
'
' History
' =======
'
' 003   Check last stream sent *before* overwriting the bit array
'       Pin allocation switched (TX+/TX-)
'       Corrected PARAM_VCFG bit shift for VMode ( major bug )
'       The bitArray now set to worst case size regardless of encoding method
'       Added 100nF to hardware interface
'
' 002   Driver initialisation now via parameter block
'       Driver parameter block initialised in Spin routines
'       Uses Baseband plus Chroma to do hardware Manchester Encoding
'       Corrected FRQA rate for softawre Manchester Encoded output
'       Corrected ETH_BYTES_MAX and ETH_BYTES_MIN values
'       Clarified TX+/TX- pin allocation
'       Improved longs to send counting
'       Added waitvid colours,#0 to force idle state after bits sent
'       Added Credits and Acknowledgements
'
' 001   First public release

' Credits and Acknowledgements
' ============================
'
' Thanks to Ariba (Andy) of the Propeller Forum for details of video generation settings.
'
' Thanks to deSilva of the Propeller Forum and his Assembly Language Tutorial tutorial for details
' of the video generation mechanism used by the Propeller Chip.
'
' Thanks to Phil Pilgrim (PhiPi) of the Propeller Forum for provision of test code to do hardware
' Manchester encoding using Broadband Video plus Chroma.
'
' Inspiration for this project came from ...
'
' http://www.fpga4fun.com/10BASE-T.html
'
'    Jean P. Nicolle : "10BASE-T FPGA interface"
'
' http://www.cesko.host.sk/IgorPlugUDP/IgorPlug-UDP%20(AVR)_eng.htm
'
'     Ing. Igor Cesko : "Implementation UDP into microcontroller Igor Atmel-UDP device: UDP/IP
'     infrared remote control"
'
' http://www.national.com/an/AN/AN-1519.pdf
'
'    National Semiconductor Application Note 1519. David Miller, Devin Seely, Tung Ngo : "DP83848
'    PHYTER Transformerless Ethernet Operation"

CON

  HARDWARE_ENABLED  = false                             ' Enable Hardware Encoding ( True or False )
  
  ETH_BYTES_MAX     = 1526                              ' Max size of Ethernet Packet in total
  ETH_BYTES_MIN     = 72

  ETH_BITS_PER_LONG = 16                                ' Worst case 16 ( 16 = S/W, 32 = H/W )
 
  ETH_BITS_MAX      = ETH_BYTES_MAX * 8
  ETH_LONGS_MAX     = ETH_BITS_MAX / ETH_BITS_PER_LONG
  
  IP_LEN_MSB        = 24
  IP_LEN_LSB        = 25

  PARAM_DIRA        = 0                                 ' DIRA Output Bits for TX+/TX-
  PARAM_CTRA        = 1                                 ' CTRA Configuration
  PARAM_FRQA        = 2                                 ' FRQA Configuration
  PARAM_VCFG        = 3                                 ' VCFG Configuration
  PARAM_VSCL        = 4                                 ' VCSL Configuration
  PARAM_16MS        = 5                                 ' CNT elapsed count for 16mS
  PARAM_NLPB        = 6                                 ' NLP bit stream ( to create NLP )
  PARAM_COLR        = 7                                 ' Bi-phase output bits ( pixel colours )
   
VAR

  long bitArray[ ETH_LONGS_MAX + 1 ]
  
  word bitArrayIndex
  long bitMask
  
  byte useHardwareEncoding
  
' *******************************************************************************************************
' *                                                                                                     *
' *     Start the Assembler Cog running                                                                 *
' *                                                                                                     *
' *******************************************************************************************************

PUB Start( ethernetBasePin )

  case ethernetBasePin
    1, 5, 9, 13, 17, 21, 25, 29 :
      useHardwareEncoding := HARDWARE_ENABLED
    other :
      useHardwareEncoding := false

  bitArray[ PARAM_DIRA ] := %11 << ethernetBasePin
  bitArray[ PARAM_16MS ] := CLKFREQ / 1000 * 16

  if useHardwareEncoding
    SetParamsForHardwareEncoding( ethernetBasePin )
  else
    SetParamsForSoftwareEncoding( ethernetBasePin )

  cognew( @BitBangedEthernetDriver, @bitArray )         ' Initialise Cog

  repeat while bitArray[0] <> 0                         ' Wait for initilisation completed
  
' *******************************************************************************************************
' *                                                                                                     *
' *     Hardware Manchester Encoding                                                                    *
' *                                                                                                     *
' *     Every bit in the byte array has been converted to one bit in the bit array, a one to one bit    *
' *     mapping. Each of those bits is to be streamed out at a rate of one bit every 100nS, 10MHz. The  *
' *     Manchester encoding is achieved with Baseband video generation using the Baseband Chroma.       *
' *                                                                                                     *
' *******************************************************************************************************

PRI SetParamsForHardwareEncoding( pin )

  bitArray[ PARAM_FRQA ] :=  1 << 29                    ' 10MHz clocking ( PLL Output = 160MHz )

  bitArray[ PARAM_CTRA ] := %00001 << 26 | %111 << 23

  ' CTRA ...
  '
  ' %-00001-- -------- -------- --------    CtrMode      000001 = Video PLL
  ' %------11 1------- -------- --------    PllDiv       111    = Divide by 1 = 160MHz
  ' %-------- -------- -XXXXXX- --------    BPin         XXXXXX = Not Used
  ' %-------- -------- -------- --XXXXXX    APin         XXXXXX = Not Used

  if pin & 4
    bitArray[ PARAM_VCFG ] := %11 << 29 | %01 << 26 | ( pin / 8 ) << 9 | %11 << ( pin & 7 )
  else
    bitArray[ PARAM_VCFG ] := %10 << 29 | %01 << 26 | ( pin / 8 ) << 9 | %11 << ( pin & 7 )

  ' VCFG ...
  '
  ' %-1X----- -------- -------- --------    VMode        10/11  = Baseband ( Low/High nibble )
  ' %---0---- -------- -------- --------    CMode        0      = Two Colour Mode
  ' %----01-- -------- -------- --------    Chroma1/0    01     = Chroma on Baseband
  ' %------00 0------- -------- --------    Aural Sub    000    = No Aural Sub Cog
  ' %-------- -------- ----XXX- --------    VGroup       XXX    = Pin / 8
  ' %-------- -------- -------- XXXXXXXX    VPins        XXX    = %11 << ( Pin & 7 )

  bitArray[ PARAM_VSCL ] := 16 << 12 | 32 * 16

  ' VSCL ...
  '
  ' %-------- ----0001 0000---- --------    PixelClocks  16     = 160MHz / 16 = 10MHz = 100nS per bit
  ' %-------- -------- ----0010 00000000    FrameClocks  512

  bitArray[ PARAM_NLPB ] := %1                          ' 1 x 100nS = 100nS NLP

  bitArray[ PARAM_COLR ] := %0001_1011_1001_1011        ' Add +/-1 to 011 = 100 & 010
  bitArray[ PARAM_COLR ] := %1011_1001_1011_1001        ' Add +/-1 to 011 = 100 & 010

' *******************************************************************************************************
' *                                                                                                     *
' *     Software Manchester Encoding                                                                    *
' *                                                                                                     *
' *     Every bit in the byte array has been converted to two bits in the bit array ( as Manchester     *
' *     encoded bit pairs ). Each of those bits is to be streamed out at a rate of one bit every 50nS,  *
' *     20MHz.                                                                                          *
' *                                                                                                     *
' *******************************************************************************************************

PRI SetParamsForSoftwareEncoding( pin )                    

  bitArray[ PARAM_FRQA ] := 1 << 28                     ' 5MHz clocking ( PLL Output = 80MHz )

  bitArray[ PARAM_CTRA ] := %00001 << 26 | %101 << 23

  ' CTRA ...
  '
  ' %-00001-- -------- -------- --------    CtrMode      000001 = Video PLL
  ' %------10 1------- -------- --------    PllDiv       101    = Divide by 4 = 20MHz
  ' %-------- -------- -XXXXXX- --------    BPin         XXXXXX = Not Used
  ' %-------- -------- -------- --XXXXXX    APin         XXXXXX = Not Used

  bitArray[ PARAM_VCFG ] := %01 << 29 | ( pin / 8 ) << 9 | %11 << ( pin & 7 )

  ' VCFG ...
  '
  ' %-01----- -------- -------- --------    VMode        01     = VGA
  ' %---0---- -------- -------- --------    CMode        0      = Two Colour Mode
  ' %----00-- -------- -------- --------    Chroma1/0    00     = No Chroma
  ' %------00 0------- -------- --------    Aural Sub    000    = No Aural Sub Cog
  ' %-------- -------- ----XXX- --------    VGroup       XXX    = Pin / 8
  ' %-------- -------- -------- XXXXXXXX    VPins        XXX    = %11 << ( Pin & 7 )

  bitArray[ PARAM_VSCL ] := 1 << 12 | 32

  ' VSCL ...
  '
  ' %-------- ----0000 0001---- --------    PixelClocks  1      = 20MHz / 1 = 20MHz = 50nS per bit
  ' %-------- -------- ----0000 00100000    FrameClocks  32     

  bitArray[ PARAM_NLPB ] := %11                         ' 2 x 50nS = 100nS NLP

  bitArray[ PARAM_COLR ] := $00_00_55_AA                ' TX- to left, TX+ to right of each two-bit pair       

  ' Even pin TX+       -+-+-+-+ -+-+-+-+               
  ' %-------- -------- -------- 10101010    Colour 0     Bit Clear ( TX- = 1, TX+ = 0 )
  ' %-------- -------- 01010101 --------    Colour 1     Bit Set   ( TX- = 0, TX+ = 1 )
  ' %-------- XXXXXXXX -------- --------    Colour 2     Not Used
  ' %XXXXXXXX -------- -------- --------    Colour 3     Not Used

  ' Odd pin TX+        +-+-+-+- +-+-+-+-
  ' %-------- -------- -------- 01010101    Colour 0     Bit Clear ( TX- = 1, TX+ = 0 )
  ' %-------- -------- 10101010 --------    Colour 1     Bit Set   ( TX- = 0, TX+ = 1 )
  ' %-------- XXXXXXXX -------- --------    Colour 2     Not Used
  ' %XXXXXXXX -------- -------- --------    Colour 3     Not Used
    
  if ( pin & 1 )                                        ' Switch TX+/TX- if odd numbered base pin ...
    bitArray[ PARAM_COLR ] ^= $00_00_FF_FF              ' ... Done by inverting the pin outputs

' *******************************************************************************************************
' *                                                                                                     *
' *     Take a byte packet, convert to bits and let the Cog stream it out                               *
' *                                                                                                     *
' *******************************************************************************************************

PUB CreateAndSendBitStream( pointerToByteArray ) | endNum, i
  
  bitArrayIndex := 0                                    ' Start with first bit
  bitMask := 0
  
  endNum := ( ( byte[ pointerToByteArray+IP_LEN_MSB ] << 8 ) | byte[ pointerToByteArray+IP_LEN_LSB ] ) + 25

  repeat until bitArray[0] == 0                         ' Wait until last has been sent

  repeat i from 0 to endNum                             ' Convert bytes
    SetBitsFromByte( byte[ pointerToByteArray+i ] )
    
  bitArray[0] := bitArrayIndex                          ' Send a new bit stream

PRI SetBitsFromByte( n )

' Converting bytes to bits is straight forward but it may not be immediately clear as to how it is done
' and how it works in total ...
'
' Each bit ( sent lsb first from the byte ) is Manchester Encoded into two bits. According to the Ethernet
' Blue-Book,a 0-bit is sent 0 first / 1 second, a 1-bit sent as 1 first / 0 second. Because the longs we
' are building will be sent lsb first, the first encoded bit is placed to the right of the second ...
'
' A byte to send : ------0-1
'                        | |
'                        | `---------------------------.
'                        `-------------------------.   |
'                                                 _|  _|
' A long to send : -------- -------- -------- ----10--01
'
' As can be seen, we can hold 16 MTU bits in each long.

' The output is also bi-phase and we use the colour registers to decide which output pins will be set and
' cleared. Each bit in the long will choose one of two colours, each colour will set one of two pins with
' the other being cleared. The two colours are opposites of each other.
'
' A byte to send : ------0-1
'                        | |
'                        | `---------------------------.
'                        `-------------------------.   |
'                                                 _|  _|
' A long to send : -------- -------- -------- ----10--01 
'                                               
' Output Pin TX+ : -------- -------- -------- ----10--01  
' Output Pin TX- : -------- -------- -------- ----01--10

  if bitMask == 0                                       ' Setup for first bit
    bitArray[++bitArrayIndex] := 0
    bitMask := 1
    
  if useHardwareEncoding                                ' Manchester Encoding done by Hardware ( Chroma )
 
    repeat 8
      if ( n & 1 ) 
        bitArray[ bitArrayIndex ] |= bitMask
      bitMask <<= 1
      n >>= 1
      
  else                                                  ' Manchester Encoding done by software
  
    repeat 8
      if ( n & 1 ) 
        bitArray[ bitArrayIndex ] |= ( bitMask      )   ' 1 -> 01
      else
        bitArray[ bitArrayIndex ] |= ( bitMask << 1 )   ' 0 -> 10
      bitMask <<= 2
      n >>= 1
      
PUB GetPointerToBitArray

  result := @bitArray[0]

DAT

' *******************************************************************************************************
' *                                                                                                     *
' *     Cog Assembler Code                                                                              *
' *                                                                                                     *
' *******************************************************************************************************

              org       0
              
BitBangedEthernetDriver

              mov       streamPtr,PAR                   ' Point to bit array

              call      #GetParam                       ' Initialise I/O pins
              mov       OUTA,#0  
              mov       DIRA,bits                       ' Make TX+ and TX- pins outputs
              
              call      #GetParam                       ' Initialise Video Hardware
              mov       CTRA,bits
              call      #GetParam
              mov       FRQA,bits
              call      #GetParam
              mov       VCFG,bits
              call      #GetParam
              mov       VSCL,bits

              call      #GetParam                       ' Timeout period for 16mS
              mov       timeout16mS,bits

              Call      #GetParam                       ' Set NLP bit stream
              mov       nlpBits,bits

              Call      #GetParam                       ' Set bi-phase output bits ( pixel colours )
              mov       colours,bits

              Call      #SendNlp                        ' Start by issuing an NLP
              
ReadyForNextStream

              mov       streamPtr,PAR                   ' Point to bit array
              wrlong    zero,streamPtr                  ' Tell sender we're ready to accept a stream

StartNlpTimeout

              mov       timeout,CNT                     ' Set timeout for 16mS +/- 8mS
              add       timeout,timeout16mS

ShortPauseAfterAnythingSent

              mov       count,#200                      ' Delay at least 9.6uS ( inter-frame gap )
Delay10uS     djnz      count,#Delay10uS                ' 200 x 50nS per instruction = 10000nS = 10uS
              
WaitForStream

              rdlong    count,streamPtr             WZ  ' Wait until stream provided ( non-zero count )
        IF_Z  jmp       #CheckForTimeout                ' Not ready yet

OutputBitStream
        
              add       streamPtr,#4                    ' Point to long to output bitArray[1..count]
              rdlong    bits,streamPtr
        
              waitvid   colours,bits
              
              djnz      count,#OutputBitStream

              waitvid   colours,#0                      ' Ensure default line state
              
              jmp       #ReadyForNextStream
              
CheckForTimeout

              ' If (timeout - CNT ) <= 0 Then TimedOut
              ' If (timeout - CNT ) >  0 Then NotTimedOut
              
              sub       timeout,CNT         NR, WC, WZ  ' Has timeout occurred ?
        IF_A  jmp       #WaitForStream                  ' No

TimedOut

              call      #SendNlp              
              jmp       #StartNlpTimeout

' *******************************************************************************************************
' *                                                                                                     *
' *     Subroutines                                                                                     *
' *                                                                                                     *
' *******************************************************************************************************
              
'             Read a paramater from the bit array during initialisation

GetParam      rdlong    bits,streamPtr                  ' Read the parameter
              add       streamPtr,#4                    ' Point to the next one
GetParam_ret  ret

'       Send a single NLP

SendNlp       waitvid   colours,#0                      ' Ensure default line state
              waitvid   colours,nlpBits                 ' Send NLP - 100nS pulse
              waitvid   colours,#0                      ' Ensure default line state
SendNlp_ret   ret

' *******************************************************************************************************
' *                                                                                                     *
' *     Predefined Constants                                                                            *
' *                                                                                                     *
' *******************************************************************************************************

zero          long      0                               ' Constant value zero

' *******************************************************************************************************
' *                                                                                                     *
' *     Run-Time Variables                                                                              *
' *                                                                                                     *
' *******************************************************************************************************

streamPtr     res       1
colours       res       1
bits          res       1
nlpBits       res       1
count         res       1
timeout       res       1
timeout16mS   res       1
tmp           res       1

              fit       $1F0
              